From 9e9b93502d984900cecde793d884c6c79c4cafc1 Mon Sep 17 00:00:00 2001
From: Niklas Haas <git@haasn.dev>
Date: Sat, 19 Nov 2022 02:00:15 +0100
Subject: [PATCH] shaders/colorspace: don't explode on luma > 1000

The current desaturation function extends linearly to infinity,
resulting in very bad behavior for extreme HDR->SDR ratios (e.g. 49:1).
Replace it by a logarithmic map to smooth this out more naturally at an
asymptote.

This is more or less what the original code was intending, I think.

Fixes: 7378526770b823cf76ca66d31ad6aa079dd15cec
---
 src/shaders/colorspace.c | 12 +++---------
 1 file changed, 3 insertions(+), 9 deletions(-)

diff --git a/src/shaders/colorspace.c b/src/shaders/colorspace.c
index 63e18336..570b8ae2 100644
--- a/src/shaders/colorspace.c
+++ b/src/shaders/colorspace.c
@@ -1463,15 +1463,9 @@ static void tone_map(pl_shader sh,
         }));
 
         // Tuned to meet the desired desaturation at 1000 <-> SDR
-        const float ref_ratio = 1000 / PL_COLOR_SDR_WHITE;
-        float ratio = src_max / dst_max, desat;
-        if (ratio >= 1) {
-            float k = (ratio - 1) / (ref_ratio - 1);
-            desat = PL_MIX(1.0f, 1/1.1f, k);
-        } else {
-            float k = (1 - ratio) / (1 - 1 / ref_ratio);
-            desat = PL_MIX(1.0f, 1.075f, k);
-        }
+        float ratio = logf(src_max / dst_max) / logf(1000 / PL_COLOR_SDR_WHITE);
+        float coeff = src_max > dst_max ? 1 / 1.1f : 1.075f;
+        float desat = (coeff - 1) * fabsf(ratio) + 1;
 
         GLSL("float orig = max(xyz.y, %s);  \n"
              "xyz.y = tone_map(xyz.y);      \n"
